/* ------------------------------------------------------------------

File:		AviationWeatherTime.js
Abstract:	time routines (sunrise/sunset, dst, local time)
Version:	1.2.10 (10/20/2008)
Author:		Pascal Dreer

------------------------------------------------------------------ */


cRad = 0.0174532925199433;
cArad = 57.2957795130823;
zenith = 90.8333333333;



function convertToLocalTime(dateObj,presetNo)
{
	var dateLT = new Date();
	
	var adjMin = parseInt(presets[presetNo].timezone,10) + ((parseFloat(presets[presetNo].timezone) - parseInt(presets[presetNo].timezone,10)) / 6 * 10);
	
	dateLT.setTime(dateObj.getTime() + (dateObj.getTimezoneOffset() * 60000) + (adjMin * 3600 * 1000));
	adjustSummertime(dateLT,presetNo,false);

	return dateLT;
}


function convertUTCtoLocal(utcDate,obj)
{
	var dateObj = new Date();

	dateObj.setHours(Math.floor(utcDate.substring(0,2)));
	dateObj.setMinutes(Math.floor(utcDate.substring(3,5)));		// 2,4


	var adjMin = ((parseFloat(obj.timezone) - parseInt(obj.timezone,10)) / 6 * 10);
	var tmptime = dateObj.getTime() + (parseInt(obj.timezone,10) * 60 * 60000) + (adjMin * 3600 * 1000);
	dateObj.setTime(tmptime);
//	alert(obj.icao + "/" + obj.timezone + "/" + dateObj.getHours()  + "/" + adjMin + "/" + parseInt(obj.timezone,10) * 60 * 60000);
	
	adjustSummertime(dateObj,obj.preset,false);

	var hours = dateObj.getHours();
	var minutes = dateObj.getMinutes();	
	
	return ((hours < 10) ? "0" + hours : hours) + ":" + ((minutes < 10) ? "0" + minutes : minutes);
}



function adjustSummertime(dateObj,presetNo,detail)
{
	var dstOn = false;
	var today = new Date();

	var curYear = today.getFullYear();			// if dst goes thru winter
	var nextYear = today.getFullYear();
	if (today.getMonth() < 5)
		curYear = curYear - 1;
	else
		nextYear = nextYear + 1;
		
	
	switch (presets[presetNo].summertime) {
		case 0: dstOn = false;					// do nothing
				break;
				
		case 1:									// europe & russia
				var dstBegin = calculateLastWeekdayOfMonth(today.getFullYear(),2,0);	// last Sun Mar
				var dateBegin = new Date(today.getFullYear(),2,dstBegin);
				
				var dstEnd = calculateLastWeekdayOfMonth(today.getFullYear(),9,0);		// last Sun Oct
				var dateEnd = new Date(today.getFullYear(),9,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 2:									// europe - iceland
//				dateObj.setTime(dateObj.getTime() + (3600 * 1000));
				dstOn = false;
				break;
				
		case 3:									// us 
				var dstBegin = calculateNWeekdayOfMonth(today.getFullYear(),2,0,2);		// first Sun Apr; NEW 2nd Sun Mar
				var dateBegin = new Date(today.getFullYear(),2,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(today.getFullYear(),10,0,1);		// last Sun Oct; NEW 1st Sun Nov
				var dateEnd = new Date(today.getFullYear(),10,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
				
		case 4:									// canada
				var dstBegin = calculateNWeekdayOfMonth(today.getFullYear(),2,0,2);		// first Sun Apr; NEW 2nd Sun Mar
				var dateBegin = new Date(today.getFullYear(),2,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(today.getFullYear(),10,0,1);		// last Sun Oct; NEW 1st Sun Nov
				var dateEnd = new Date(today.getFullYear(),10,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 5:									// us NOT USED
				break;

		case 6:									// chile
				var dstBegin = calculateNWeekdayOfMonth(curYear,9,0,2);		// 2nd Sun Oct
				var dateBegin = new Date(curYear,9,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,2,0,2);		// 2nd Sun Mar
				var dateEnd = new Date(nextYear,2,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 7:									// brazil
				switch (curYear) {
					case 2008:	var dateBegin = new Date(curYear,9,21);						// 21 Oct
								var dateEnd = new Date(nextYear,1,16);						// 16 Feb
								break;
					case 2009:	var dateBegin = new Date(curYear,9,19);						// 19 Oct
								var dateEnd = new Date(nextYear,1,14);						// 14 Feb
								break;
					default:	var dateBegin = new Date(curYear,9,21);						// use 2008
								var dateEnd = new Date(nextYear,1,16);
								break;
				}

				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 8:									// australia 1 New South Wales, Victoria, South Australia, Tasmania, ACT
				var dstBegin = calculateNWeekdayOfMonth(curYear,9,0,1);		// 1st Sun Oct
				var dateBegin = new Date(curYear,9,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,3,0,1);	// 1st Sun Apr
				var dateEnd = new Date(nextYear,3,dstEnd);
				
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 9:									// australia 2 Western Australia
				var dstBegin = calculateLastWeekdayOfMonth(curYear,9,0);	// last Sun Oct
				var dateBegin = new Date(curYear,9,dstBegin);
				
				var dstEnd = calculateLastWeekdayOfMonth(nextYear,2,0);		// last Sun Mar
				var dateEnd = new Date(nextYear,2,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 10:								// new zealand
				var dstBegin = calculateLastWeekdayOfMonth(curYear,8,0);	// last Sun Sep
				var dateBegin = new Date(curYear,8,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,3,0,1);	// 1st Sun Apr
				var dateEnd = new Date(nextYear,3,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 11:								// iran
				var dateBegin = new Date(today.getFullYear(),2,22);						// 22 Mar
				
				var dateEnd = new Date(today.getFullYear(),8,22);						// 22 Sep

				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 12:								// israel
				switch (today.getFullYear()) {
					case 2008:	var dateBegin = new Date(today.getFullYear(),2,28);						// 28 Mar
								var dateEnd = new Date(today.getFullYear(),9,5);						// 5 Oct
								break;
					case 2009:	var dateBegin = new Date(today.getFullYear(),2,27);						// 27 Mar
								var dateEnd = new Date(today.getFullYear(),9,27);						// 27 Oct
								break;
					default:	var dateBegin = new Date(2006,2,31);									// use 2006
								var dateEnd = new Date(2006,9,1);
								break;
				}

				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 13:								// egypt
				var dstBegin = calculateLastWeekdayOfMonth(today.getFullYear(),3,5);	// last Fri Apr
				var dateBegin = new Date(today.getFullYear(),3,dstBegin);
				
				var dstEnd = calculateLastWeekdayOfMonth(today.getFullYear(),8,4);		// last Thu Sep
				var dateEnd = new Date(today.getFullYear(),8,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		case 14:								// russia -- not used
				break;

		case 15:								// uruguay
				var dstBegin = calculateNWeekdayOfMonth(today.getFullYear(),2,0,2);		// 2nd Sun Mar
				var dateBegin = new Date(today.getFullYear(),2,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(today.getFullYear(),9,0,2);		// 2nd Sun Oct
				var dateEnd = new Date(today.getFullYear(),9,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		case 16:								// syria
				var dateBegin = new Date(today.getFullYear(),3,1);						// 1 Apr
				
				var dateEnd = new Date(today.getFullYear(),8,30);						// 30 Sep

				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 17:								// namibia
				var dstBegin = calculateNWeekdayOfMonth(curYear,8,0,1);		// 1st Sun Sep
				var dateBegin = new Date(curYear,8,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,3,0,1);		// 1st Sun Apr
				var dateEnd = new Date(nextYear,3,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		case 18:								// jordan
				var dstBegin = calculateLastWeekdayOfMonth(today.getFullYear(),2,4);	// last Thu Mar
				var dateBegin = new Date(today.getFullYear(),2,dstBegin);
				
				var dstEnd = calculateLastWeekdayOfMonth(today.getFullYear(),8,5);		// last Fri Sep
				var dateEnd = new Date(today.getFullYear(),8,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 19:								// falkland islands
				var dstBegin = calculateNWeekdayOfMonth(curYear,8,0,1);	// 1st Sun Sep
				var dateBegin = new Date(curYear,8,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,3,0,3);		// 3rd Sun Apr
				var dateEnd = new Date(nextYear,3,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		case 20:								// paraguay
				var dstBegin = calculateNWeekdayOfMonth(curYear,9,0,3);	// 3nd Sun Oct
				var dateBegin = new Date(curYear,9,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,2,0,2);		// 2nd Sun Mar
				var dateEnd = new Date(nextYear,2,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		case 21:								// Saint Pierre and Miquelon & Turks and Caicos Islands
				var dstBegin = calculateNWeekdayOfMonth(today.getFullYear(),3,0,1);		// 1nd Sun Apr
				var dateBegin = new Date(today.getFullYear(),3,dstBegin);
				
				var dstEnd = calculatelastWeekdayOfMonth(today.getFullYear(),9,0);		// last Sun Oct
				var dateEnd = new Date(today.getFullYear(),9,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;
				
		case 22:								// argentina
				var dstBegin = calculateNWeekdayOfMonth(curYear,9,0,3);		// 3rd Sun Oct
				var dateBegin = new Date(curYear,9,dstBegin);
				
				var dstEnd = calculateNWeekdayOfMonth(nextYear,2,0,3);		// 3rd Sun Mar
				var dateEnd = new Date(nextYear,2,dstEnd);
				
				if ((today.getTime() > dateBegin.getTime()) && (today.getTime() < dateEnd.getTime())) {
					dateObj.setTime(dateObj.getTime() + (3600 * 1000));					// add 1 hour
					dstOn = true;
				}
				break;

		default:
				break;
	}
	
	if (detail) {
		if (dstOn)
			var dstStr1 = ",<br>" + getLocalizedString("currently observed");
		else
			var dstStr1 = ",<br>" + getLocalizedString("currently not observed");
			
		if (dateBegin) {
			if (metricFormat >= 3) {
				var dstStr2 = dateBegin.getDate() + "." + (dateBegin.getMonth() + 1.0) + "." + dateBegin.getFullYear() + " - " +
							 dateEnd.getDate() + "." + (dateEnd.getMonth() + 1.0) + "." + dateEnd.getFullYear();
			}
			else {
				var dstStr2 = (dateBegin.getMonth() + 1.0) + "/" + dateBegin.getDate() + "/" + dateBegin.getFullYear() + " - " +
							  (dateEnd.getMonth() + 1.0) + "/" + dateEnd.getDate() + "/" + dateEnd.getFullYear();
			}
		}
		else {
//			if (presets[presetNo].summertime == 0) {
				var dstStr2 = getLocalizedString("no Summertime");
				var dstStr1 = "";
/*			}
			else {
				var dstStr1 = "";
				var dstStr2 = "";
			}
*/		}
		return dstStr2 + dstStr1;
	}
	else
		return dstOn;
}


function calculateLastWeekdayOfMonth(year,month,weekday)
{
	var tmpDate = new Date(year,month+1,1,0,0,0);
	tmpDate.setTime(tmpDate.getTime() - 1000);
	var endDay = tmpDate.getDate();
	
	for (var i=endDay;i>0;i--) {
		tmpDate.setDate(i);
		if (tmpDate.getDay() == weekday)
			break;
	}

	return i;
}

function calculateNWeekdayOfMonth(year,month,weekday,n)
{
	var tmpDate = new Date(year,month,1,0,0,0);
	var j = 0;
	
	for (var i=1;i<28;i++) {
		tmpDate.setDate(i);
		if (tmpDate.getDay() == weekday) {
			j++;
			if (j == n)
				break;
		}
			
	}

	return i;
}



function calculateWeekdaysPerMonth(year,month,weekday)	// NOT USED
{
	var numOfWeekdays = 0;
	
	var tmpDate = new Date(year,month+1,1,0,0,0);
	tmpDate.setTime(tmpDate.getTime() - 1000);
	var endDay = tmpDate.getDate();
	
	for (var i=1;i<=endDay;i++) {
		tmpDate.setDate(i);
		if (tmpDate.getDay() == weekday)
			numOfWeekdays++;
	}
	
	return numOfWeekdays;
}




function calculateSunrise(date,latitude,longitude,sunrise)
{
	// based to: Almanac for Computers, 1990; published by Nautical Almanac Office

	var n1 = Math.floor(275 * (date.getMonth()+1) / 9);			// get day of year
	var n2 = Math.floor(((date.getMonth()+1) + 9) / 12);
	var n3 = (1 + Math.floor((date.getFullYear() - 4 * Math.floor(date.getFullYear() / 4) + 2) / 3));
	var n = n1 - (n2 * n3) + date.getDate() - 30;

	
	var lngHour = longitude / 15;
	if (sunrise)
		var t = n + ((6 - lngHour) / 24);
	else
		var t = n + ((18 - lngHour) / 24);

	var m = (0.9856 * t) - 3.289;
	
	var l = m + (1.916 * Math.sin(m * cRad)) + (0.020 * Math.sin(2 * m * cRad)) + 282.634;
	if (l < 0)
		l = l + 360;
	if (l > 360)
		l = l - 360;
		
	var ra = Math.atan(0.91764 * Math.tan(l * cRad)) * cArad;
	if (ra < 0)
		ra = ra + 360;
	if (ra > 360)
		ra = ra - 360;
		
	var lquad = (Math.floor(l / 90)) * 90;
	var raquad = (Math.floor(ra / 90)) * 90;
	ra = ra + (lquad - raquad);
	ra = ra / 15;
	
	var sinDec = 0.39782 * Math.sin(l * cRad);
	var cosDec = Math.cos((Math.asin(sinDec) * cArad) * cRad);
	
	
	var cosH = (Math.cos(zenith * cRad) - (sinDec * Math.sin(latitude * cRad))) / (cosDec * Math.cos(latitude * cRad));
	if (cosH > 1) {
		return 0;
	}
	if (cosH < -1) {
		return 0;
	}
	

	if (sunrise)
		var h = 360 - (Math.acos(cosH) * cArad);
	else
		var h = Math.acos(cosH) * cArad;
	h = h / 15;
	
	var t = h + ra - (0.06571 * t) - 6.622;
	var ut = t - lngHour;
	
	if (ut >= 24)
		ut = ut - 24;
	if (ut < 0)
		ut = ut + 24;
	
	var hrs = Math.floor(ut);
	var mins = Math.floor((ut - hrs) * 60);
	
	date.setHours(hrs);
	date.setMinutes(mins);

	var timeTmp = date.getTime() - (date.getTimezoneOffset() * 60000);
	date.setTime(timeTmp);

	
	return hrs + ":" + mins;
}